program Potential3D
  use sep
  use cgstep_mod
  use laplac3
  use igrad1
  use solver_reg_mod
  use mask1
  use util_
  implicit none
  type(myaxis) :: ax,ay
  integer                               :: nz,iix,iiy,iiz,ii,jj,stat,iz
  integer                               :: n1,n2,n3,niter,oz,dz,xpad,ypad
  real                                  :: dfill,ifill,depth,ox,oy
  real,    dimension(:    ), allocatable:: xcoord,ycoord,zcoord,xin,yin
  real   , dimension(:    ), allocatable:: Potential,nulldata
  real   , dimension(:,:  ), allocatable:: zin,zsurf
  real,    dimension(:,:,:), allocatable:: data,model
  logical, dimension(:    ), pointer    :: msk
  logical, dimension(:,:,:), allocatable:: mfixed

  call sep_init()
  call hinaxis(ax,1)
  call hinaxis(ay,2)
  call printaxis(ax)
  call printaxis(ay)

  call from_param("nz",nz,100)
  call from_param("depth",depth,0.)
  call from_param("niter",niter,20)
  call from_param("xpad",xpad,40)
  call from_param("ypad",ypad,40)
  write(0,*) 'READ IN PARAMETERS'

  !! . . Pad data space
  n1=nz
  n2=ax%n+2*xpad
  n3=ay%n+2*ypad
  write(0,*) 'OUTPUT MODEL SPACE',n1,n2,n3
  write(0,*) 'PADDED DATA SPACE'

  allocate( zcoord(n1),xcoord(n2),ycoord(n3), mfixed(n1,n2,n3) )
  allocate( xin(ax%n),yin(ay%n),zin(ax%n,ay%n),data(n1,n2,n3),model(n1,n2,n3) )
  allocate( Potential(n1*n2*n3),msk(n1*n2*n3),nulldata(n1*n2*n3),zsurf(n2,n3));
  write(0,*) 'ALLOCATED MEMORY'

  call sep_read( zin )
  zin=zin+depth
  xin=(/ (ii,ii=0,ax%n-1) /)*ax%d+ax%o
  yin=(/ (ii,ii=0,ay%n-1) /)*ay%d+ay%o
  write(0,*) 'READ IN DATA'

  !! . .Fix Z-surf
  zsurf(xpad+1:ax%n+xpad,ypad+1:ay%n+ypad)=zin
  do ii=1,ay%n
     zsurf(        1:xpad,ypad+ii )=zin(1 ,ii)
     zsurf(ax%n+xpad+1:n2,ypad+ii )=zin(ax%n,ii) 
  end do
  do ii=1,ax%n
     zsurf( xpad+ii,1:ypad     )=zin(ii,1)
     zsurf( xpad+ii,ay%n+ypad+1)=zin(ii,ay%n) 
  end do
  zsurf(1:xpad        ,1:ypad        )=zsurf(xpad+1   ,ypad+1   )
  zsurf(1+xpad+ax%n:n2,1:ypad        )=zsurf(xpad+ax%n,ypad+1   )
  zsurf(1:xpad        ,1+ypad+ay%n:n3)=zsurf(xpad+1   ,ypad+ay%n)
  zsurf(1+xpad+ax%n:n2,1+ypad+ay%n:n3)=zsurf(xpad+ax%n,ypad+ay%n)

  oz= maxval(zsurf)
  dz=oz/(real(n1)-1.)
  zcoord=(/ (ii,ii=0,nz-1) /)*dz+oz
  write(0,*) 'FIXED SURFACE'

  !! . . Fix X-coordinates
  ox=xin(1)-xpad*ax%d
  xcoord(1:xpad)=(/ (ii,ii=0,xpad-1) /)*ax%d+ox
  xcoord(xpad+1:ax%n+xpad)=xin
  xcoord(ax%n+xpad+1:n2)= (/ (ii,ii=ax%n+xpad+1,n2) /)*ax%d+ox

  !! . . Fix Y-coordinates
  oy=yin(1)-ypad*ay%d
  ycoord(1:ypad)=(/ (ii,ii=0,ypad-1) /)*ay%d+oy
  ycoord(ypad+1:ay%n+ypad)=yin
  ycoord(ay%n+ypad+1:n3)= (/ (ii,ii=ay%n+ypad+1,n3) /)*ay%d+oy
  write(0,*) 'FIXED COORDINATES'

  mfixed=.true.
  do ii=1,n2
     do jj=1,n3
        !! . . Top Layer
        iix =floor( (xcoord(ii)-ox)/ax%d + 1.5 )
        iiy =floor( (ycoord(jj)-oy)/ay%d + 1.5 )
        if (iix .lt. 1 ) iix=1
        if (iiy .lt. 1 ) iiy=1
        if (iix .gt. n2) iix=n2
        if (iiy .gt. n3) iiy=n3
        iiz =floor( (oz-zsurf(iix,iiy))/dz + 1.5 )
        if (iiz .lt. 1)  iiz=1
        if (iiz .gt. n3) iiz=n3

        !! . . Fix Boundaries
        data  (1:iiz,iix,iiy)=1.
        mfixed(1:iiz,iix,iiy)=.false.
        dfill=1./real(n1-iiz+1)
        ifill=1.
        do iz=iiz+1,n1-1
           data(iz,ii,jj)= 1.-ifill*dfill
           ifill=ifill+1.
        end do
     end do
  end do
  data  (n1,:,:)=0.;
  mfixed(n1,:,:)=.false.
  write(0,*) 'SET UP INITIAL DATA MATRIX WITH MASK'

  !! . . Deallocate temporary arrays
  deallocate( zcoord,ycoord,xcoord,zin,xin,yin)

  !! . . Call the laplacian filler
  Potential = reshape(data  ,(/n1*n2*n3/))
  msk       = reshape(mfixed,(/n1*n2*n3/)) 

  write(0,*) 'STARTING LAPLACIAN SOLVER'
  write(0,*) 'NUMBER OF ITERATIONS: ',niter
  call mask1_init(msk)
  call laplac3_init (n1,n2,n3)
  call solver_reg( m=Potential, d=nulldata, Fop=laplac3_lop, stepper=cgstep,&
  niter=niter, m0=Potential, Jop=mask1_lop,Aop=laplac3_lop,naop=n1*n2*n3,eps=0.,&
  verb=.true.)

  model=reshape(Potential,(/n1,n2,n3/) )
  call to_history("n1",n1)
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("o1",oz-depth)
  call to_history("o2",ox)
  call to_history("o3",oy)
  call to_history("d1",-dz)
  call to_history("d2",ax%d)
  call to_history("d3",ay%d)
  call to_history("esize",4)
  call hclose()

  write(0,*) 'OUTPUTING MODEL FILE'

  call sep_write(model)

  call laplac3_close()
  call cgstep_close ()
  call igrad1_close()
  deallocate(mfixed,data,model,Potential,msk,nulldata)
  call exit()
end program Potential3D
